# ridf2root
This convert .ridf to .root file

## Environment
* ROOT v6.14 (need to set environment variable ROOTSYS)
  * (M1 Mac cannot install old version of ROOT (okawa) )
* cmake 3.8 and above
* yaml-cpp
* some package(wget, automake autoconf...)
* (optionally python and pip pyyaml)

## Summary
This is a tool to convert ridffile to rootfile using ANAROOT decoder. (https://ribf.riken.jp/RIBFDAQ/index.php?Tools%2FAnalysis%2FANAROOT)

ANAROOT is automatically installed by "cmake" in the project directory in order to make it easier to edit decoder file. The part of install of ANAROOT is written in ./CMakeLists.txt.

## Note
* Multihit TDC records all hit in one trigger, but map tree include only one hit event. (last hit)
  * raw data tree store all hit and calibration tree also contain all hit
  * the format of timing branch of calibration tree are array (like branch[12]) in order to match the format of mhtdc

* The structure of map tree branches depends on the map file, so now a single branch contains both ADC and TDC data and it is distinguished by channel.
  * ex. "name[0]": ADC, "name[1]": TDC

* When the name contains "-", "/" or " " in mapfile and we use it as branch name, sometimes it is recognized as an operator, so this tool change them to "\_".

* Calibration tree depends on the map tree. And the name of parameter file is important (cannot name arbitrary)

* Assume that calibration parameters of two energy output are the same

## Usage

### Config file

* recipe/**.yml
* map/(expname)/**.map
* calibration/(expname)/**.prm
* main.cpp

#### YAML file


#### Map file


#### Calibration parameter file


### Install
This is cmake based project, so make build directory and compile in it.

```console
mkdir build
cd build
cmake ..
```

After this, ANAROOT will be installed in project directory and we can see ./anaroot directory. In order to read this anaroot environment, we needed to read ./anaroot/setup.sh

```console
source ../anaroot/setup.sh
```
And compile
```console
make
```

If it works, an executable file "ridf2root" will be created, and it will be executed in the following format.
```console
./ridf2root -i ridfname (-o rootname)
```

When the -o option is not written, the name of the output rootfile is the same as ridffile.

If something goes wrong, redo the build and sometimes it will work.
```console
cd ..
rm -rf build
mkdir build && cd build
cmake ..
(source ../anaroot/setup.sh)
make
```




