#ifndef _TCORDFERBRANCHMANAGER_HH_
#define _TCORDFERBRANCHMANAGER_HH_

#include <vector>
#include <array>
#include <unordered_map>
#include <yaml-cpp/yaml.h>

#include <TFile.h>
#include <TTree.h>
#include <TH1.h>

#include "ChannelMap.hh"
#include "CalibParamReader.hh"
#include "TCordferRawBranch.hh"
#include "TCordferMapBranch.hh"
#include "TCordferCalibBranch.hh"

using namespace std;

class TCordferBranchManager
{
  private:
    TFile *rootfile;
    TString root_name;
    TString ridf_name;
    YAML::Node recipe;

    TTree *tree;
    TTree *mtree;
    TTree *ctree;
    TH1I *scaler;

    vector<TCordferRawBranch*> branch;
    vector<TCordferMapBranch*> mbranch;
    vector<TCordferCalibBranch*> cbranch;
    unordered_map<Int_t, Int_t> mapping;
    unordered_map<Int_t, Int_t> mmapping;
    unordered_map<Int_t, Int_t> cmapping;
    unordered_map<Int_t, Int_t> muxmapping;

    vector< vector<Double_t> > mpos; //for mssd

    //for temporary
    bool rmapflag = false;
    bool mapflag = false;
    bool cmapflag = false;

  public:
    TCordferBranchManager(){}
    TCordferBranchManager(TString input, TString output){
      ridf_name = input;
      root_name = output;
    }
    ~TCordferBranchManager(){}

    TString get_ridf_name(){ return ridf_name; }
    TString get_root_name(){ return root_name; }

    void Readyaml(TString yamlfile);
    void CreateRootFile(){ rootfile = new TFile(root_name.Data(), "recreate"); }
    void CreateRawTree();
    void CreateMapTree();
    void CreateCalibTree();
    void CreateRawBranch();
    void CreateMapBranch();
    void CreateCalibBranch();
    void CreateRawMapping();
    void CreateMapping();
    void CreateCalibMapping();
    void GiveRawValue(Int_t geo, Int_t ch, Int_t val);
    void GiveMapValue(Int_t geo, Int_t ch, Int_t val);
    void GiveCalibValue(Int_t geo, Int_t ch, Int_t val);
    void GiveWaitCalibValue();
    void RawFill(){ tree->Fill(); }
    void MapFill(){ mtree->Fill(); }
    void CalibFill() { ctree->Fill(); }
    void ClearRawAll();
    void ClearMapAll();
    void ClearCalibAll();
    void RawDataWrite(){ tree->Write(); }
    void DataWrite(){ mtree->Write(); }
    void CalibDataWrite(){ ctree->Write(); }
    void FileClose();

    void DEBUG_PRINT();
};

#endif
