#include <iostream>
#include <array>
#include <yaml-cpp/yaml.h>

#include <TArtEventStore.hh>
#include <TArtRawEventObject.hh>
#include <TArtCore.hh>

#include "CommandlineReader.hh"
#include "TCordferBranchManager.hh"


using namespace std;

Int_t main(Int_t argc, Char_t **argv)
{
  //map tree is require to make calibration tree
  bool rawdata = true;
  bool mapdata = true;
  bool calibration = true;

  if(mapdata == false && calibration == true){
    cout << "ERROR: calibration tree require to use map tree" << endl;
    cout << "\tCHANGE mapdata = true" << endl;
    return 0;
  }

  bool print_progress = true;
  //prepare recipe.yml (assume that execute in build directory)
  TString yamlfile = "../recipe/recipe_si26a.yml";

  //check the input and input the filename from command line
  //filename[0]: input file, filename[1]: output file
  array<TString, 2> filename = CheckCommandLine(argc, argv);

  //prepare and input the rootfile
  TCordferBranchManager *branch_manager = new TCordferBranchManager(filename[0], filename[1]);
  branch_manager->Readyaml(yamlfile);
  branch_manager->CreateRootFile();
  if(rawdata){ branch_manager->CreateRawTree(); }
  if(mapdata){ branch_manager->CreateMapTree(); }
  if(calibration){ branch_manager->CreateCalibTree(); }

  //prepare branch
  if(rawdata){ branch_manager->CreateRawBranch(); }
  if(mapdata){ branch_manager->CreateMapBranch(); }
  if(calibration){ branch_manager->CreateCalibBranch(); }

  //map segment_id and branch
  if(rawdata){ branch_manager->CreateRawMapping(); }
  if(mapdata){ branch_manager->CreateMapping(); }
  if(calibration){ branch_manager->CreateCalibMapping(); }

  //start to decode using anaroot
  TArtEventStore *estore = new TArtEventStore();
  estore->Open(filename[0].Data());
  //Int_t runnumber = estore->GetRunInfo()->GetRunNumber().Atoi();
  TArtRawEventObject *rawevent = estore->GetRawEventObject();

  Int_t evtn = 0;
  while(estore->GetNextEvent()){
    if(rawdata){ branch_manager->ClearRawAll(); }
    if(mapdata){ branch_manager->ClearMapAll(); }
    if(calibration){ branch_manager->ClearCalibAll(); }

    //eventloop
    for(Int_t i=0; i<rawevent->GetNumSeg(); i++){
      TArtRawSegmentObject *seg = rawevent->GetSegment(i);
      Int_t size = seg->GetNumData();

      for(Int_t j=0; j<size; j++){
        TArtRawDataObject *d = seg->GetData(j);
        Int_t geo = d->GetGeo();
        Int_t ch = d->GetCh();
        Int_t val = d->GetVal();

        if(rawdata){ branch_manager->GiveRawValue(geo, ch, val); }
        if(mapdata){ branch_manager->GiveMapValue(geo, ch, val); }
        if(calibration){ branch_manager->GiveCalibValue(geo, ch, val); }
      }
    }
    // for process of MUX
    if(calibration){ branch_manager->GiveWaitCalibValue(); }

    if(rawdata){ branch_manager->RawFill(); }
    if(mapdata){ branch_manager->MapFill(); }
    if(calibration){ branch_manager->CalibFill(); }

    estore->ClearData();
    if(print_progress && evtn%10000 == 0) { cout << "reading..." << evtn << endl; }
    evtn++;
  }

  if(rawdata){ branch_manager->RawDataWrite(); }
  if(mapdata){ branch_manager->DataWrite(); }
  if(calibration){ branch_manager->CalibDataWrite(); }
  branch_manager->FileClose();

  return 0;
}
