#include <iostream>
#include <array>
#include <stdlib.h>
#include <unistd.h>

#include <TSystem.h>

#include "CommandlineReader.hh"

using namespace std;

void Usage(Char_t *argv0)
{
  cout << "[decoder]: Usage: \n"
       << "\t" << argv0 << " -i [input_ridf_file_name]\n"
       << "\t-o [output_root_file_name] (default: [input_ridf_file_name].root)\n\n"
       << "\tNOTE: please don't include .ridf or .root in the command line"
       << endl;
}

array<TString, 2> CheckCommandLine(Int_t argc, Char_t **argv)
{
  TString finname = "default";
  TString foutname = "default";

  if(argc < 3){
    Usage(argv[0]);
    exit(0);
  }
  Int_t opt = 0;
  const Char_t *optstring = "i:o:";

  while((opt = getopt(argc, argv, optstring)) != -1){
    if(opt == 'i'){
      finname = optarg;
    }else if(opt == 'o'){
      foutname = optarg;
    }else{
      Usage(argv[0]);
      exit(0);
    }
  }

  if(finname == "default"){
    cerr << "EROOR: No input file" << endl;
    Usage(argv[0]);
    exit(0);
  }
  if(foutname == "default"){
    foutname = finname;
  }

  finname += ".ridf";
  foutname += ".root";

  FileStat_t info;
  if(gSystem->GetPathInfo(finname.Data(), info) != 0){
    cerr << "EROOR: File '" << finname.Data() << "' does not exist." << endl;
    cerr << "NOTE : please don't include .ridf or .root in the command line" << endl;
    exit(0);
  }

  cout << "\tinput file name  : " << finname << endl;
  cout << "\toutput file name : " << foutname << endl;

  return {finname, foutname};
}
